import React, { useState, useEffect, Fragment, useRef, useCallback, Suspense, useTransition } from 'react';
import { Routes, Route, useLocation } from 'react-router-dom';
import { Copy, Sparkles, AlertCircle, Lock, Check, AlignLeft as AlignTextLeft, Smile, Clock, Zap, Key } from 'lucide-react';
import { Dialog, Transition } from '@headlessui/react';
import { cleanDashesWithAI } from './utils/textCleaner';
import { useWordLimits } from './utils/wordLimitManager';
import { APP_NAME, UI, SAMPLE_TEXT, DEFAULT_OPTIONS, WORD_LIMITS } from './config/settings';
import { useFingerprint } from './hooks/useFingerprint';
import { PremiumFeatureButton } from './components/PremiumFeatureButton';
import { PaymentModal, UnlockModal, PremiumFeatureModal, FAQ, Footer, ComingSoon } from './components/LazyComponents';
import { SuccessPage } from './pages/SuccessPage';
import { TermsPage } from './pages/TermsPage';
import { PrivacyPage } from './pages/PrivacyPage';
import { AdminPage } from './pages/AdminPage';
import { LoginPage } from './pages/LoginPage';
import { ProtectedRoute } from './components/ProtectedRoute';
import { supabase } from './utils/supabaseClient';
import { ClipboardOverlay } from './components/ClipboardOverlay';
import ProductHuntBadge from './components/badgeHunder';
import { getStoredAccessCode, storeAccessCode, removeAccessCode, isValidStoredAccess } from './utils/accessCodeManager';

const LoadingSpinner = () => (
  <div className="flex items-center justify-center p-4">
    <div className="animate-spin rounded-full h-8 w-8 border-b-2 border-gray-900"></div>
  </div>
);

const AutoResizingTextarea: React.FC<{
  value: string;
  onChange?: (e: React.ChangeEvent<HTMLTextAreaElement>) => void;
  className?: string;
  placeholder?: string;
  maxLength?: number;
  spellCheck?: boolean;
  autoComplete?: string;
  autoCorrect?: string;
  autoCapitalize?: string;
  readOnly?: boolean;
}> = ({ value, onChange, className = '', ...props }) => {
  const textareaRef = useRef<HTMLTextAreaElement>(null);

  const adjustHeight = useCallback(() => {
    const textarea = textareaRef.current;
    if (!textarea) return;

    textarea.style.height = 'auto';
    
    const newHeight = Math.max(320, textarea.scrollHeight);
    textarea.style.height = `${newHeight}px`;
  }, []);

  useEffect(() => {
    adjustHeight();
  }, [value, adjustHeight]);

  useEffect(() => {
    window.addEventListener('resize', adjustHeight);
    return () => window.removeEventListener('resize', adjustHeight);
  }, [adjustHeight]);

  return (
    <textarea
      ref={textareaRef}
      value={value}
      onChange={onChange}
      className={`transition-height duration-200 ease-in-out ${className}`}
      {...props}
    />
  );
};

const ProcessingBanner = ({ onUpgrade }: { onUpgrade: () => void }) => (
  <div className="fixed inset-x-0 bottom-0 z-50">
    <div className="max-w-xl mx-auto p-4">
      <div className="bg-white rounded-lg shadow-xl overflow-hidden">
        <div className="h-1 bg-gradient-to-r from-indigo-500 via-purple-500 to-pink-500" />
        <div className="p-4 flex items-center justify-between">
          <div className="flex items-center space-x-3">
            <div className="flex-shrink-0">
              <Clock className="h-5 w-5 text-purple-500 animate-spin" />
            </div>
            <div>
              <p className="text-sm font-medium text-gray-900">Processing your text...</p>
              <p className="text-xs text-gray-500">Free users experience a short delay</p>
            </div>
          </div>
          <button
            onClick={onUpgrade}
            className="flex items-center px-3 py-1.5 text-sm font-medium text-white bg-gradient-to-r from-indigo-500 via-purple-500 to-pink-500 rounded-md hover:opacity-90 transition-opacity"
          >
            <Zap className="h-4 w-4 mr-1.5" />
            Upgrade for instant results
          </button>
        </div>
      </div>
    </div>
  </div>
);

function MainApp() {
  const [isPending, startTransition] = useTransition();
  const [input, setInput] = useState(SAMPLE_TEXT);
  const [output, setOutput] = useState('');
  const [wordCount, setWordCount] = useState(0);
  const [copied, setCopied] = useState(false);
  const [isProcessing, setIsProcessing] = useState(false);
  const [modalOpen, setModalOpen] = useState(false);
  const [modalType, setModalType] = useState<'daily' | 'words' | 'feature'>('daily');
  const [autoCopyEnabled, setAutoCopyEnabled] = useState(DEFAULT_OPTIONS.AUTO_COPY);
  const [removeEmojisEnabled, setRemoveEmojisEnabled] = useState(DEFAULT_OPTIONS.REMOVE_EMOJIS);
  const [isPro, setIsPro] = useState(false);
  const [isPaymentModalOpen, setPaymentModalOpen] = useState(false);
  const [isUnlockModalOpen, setIsUnlockModalOpen] = useState(false);
  const [isPremiumModalOpen, setIsPremiumModalOpen] = useState(false);
  const [selectedFeature, setSelectedFeature] = useState('');
  const { fingerprint } = useFingerprint();
  const [codeInfo, setCodeInfo] = useState<{
    code: string;
    deviceCount: number;
    activationDate?: string;
    email?: string;
  } | null>(null);
  const [showClipboardOverlay, setShowClipboardOverlay] = useState(false);
  const [documentHidden, setDocumentHidden] = useState(false);
  const [isLoading, setIsLoading] = useState(false);
  const [error, setError] = useState<string | null>(null);
  const [code, setCode] = useState('');
  const [success, setSuccess] = useState(false);
  const [activeCode, setActiveCode] = useState('');
  const [deviceCount, setDeviceCount] = useState(0);

  const {
    wordsUsedToday,
    dailyWordLimit,
    perUseWordLimit,
    addProcessedWords,
    checkWordLimits,
  } = useWordLimits(isPro);

  useEffect(() => {
    const handleVisibilityChange = () => {
      if (document.hidden) {
        setDocumentHidden(true);
      } else if (documentHidden) {
        setShowClipboardOverlay(true);
        setDocumentHidden(false);
      }
    };

    document.addEventListener('visibilitychange', handleVisibilityChange);
    return () => document.removeEventListener('visibilitychange', handleVisibilityChange);
  }, [documentHidden]);

  useEffect(() => {
    const storedAccess = getStoredAccessCode();
    if (storedAccess) {
      setIsPro(storedAccess.type === 'pro');
    }
  }, []);

  useEffect(() => {
    const validateAccess = async () => {
      if (!fingerprint) return;

      const storedAccess = getStoredAccessCode();
      if (!storedAccess || !isValidStoredAccess(fingerprint)) {
        setIsPro(false);
        removeAccessCode();
        return;
      }

      try {
        const { data: useResult, error: useError } = await supabase.rpc('use_access_code', {
          p_code: storedAccess.code,
          p_fingerprint: fingerprint
        });

        if (useError || !useResult?.success) {
          console.error('Error validating access:', useError || 'Invalid response');
          setIsPro(false);
          removeAccessCode();
          return;
        }

        storeAccessCode({
          code: storedAccess.code,
          type: useResult.type,
          credits: useResult.credits
        }, fingerprint);

        setIsPro(useResult.type === 'pro');

        const { data: codeData } = await supabase
          .from('access_codes')
          .select('fingerprints_used,created_at,metadata')
          .eq('code', storedAccess.code)
          .maybeSingle();

        if (codeData) {
          const fingerprintsCount = Array.isArray(codeData.fingerprints_used) 
            ? codeData.fingerprints_used.length 
            : 0;

          setCodeInfo({
            code: storedAccess.code,
            deviceCount: fingerprintsCount,
            activationDate: codeData.created_at,
            email: codeData.metadata?.customer_email
          });
        }
      } catch (err) {
        console.error('Error validating access code:', err);
        setIsPro(false);
        removeAccessCode();
      }
    };

    validateAccess();
  }, [fingerprint]);

  const handleSubmit = async (e: React.FormEvent) => {
    e.preventDefault();
    
    if (!fingerprint) {
      setError('Unable to verify your browser. Please try again.');
      return;
    }

    setIsLoading(true);
    setError(null);

    try {
      const { data, error: rpcError } = await supabase.rpc('use_access_code', {
        p_code: code.trim(),
        p_fingerprint: fingerprint
      });

      if (rpcError) throw new Error(rpcError.message);

      if (!data.success) {
        setError(data.error);
        return;
      }

      storeAccessCode({
        code: code.trim(),
        type: data.type,
        credits: data.credits
      }, fingerprint);

      setActiveCode(code);
      setDeviceCount(1);
      setSuccess(true);
      
      setTimeout(() => {
        onUnlock(data.type, data.credits);
        onClose();
      }, 1500);
    } catch (err) {
      setError(err instanceof Error ? err.message : 'Failed to validate code');
    } finally {
      setIsLoading(false);
    }
  };

  const handleProcess = async () => {
    if (!input.trim()) return;

    const words = input.trim().split(/\s+/).length;
    
    if (!isPro) {
      const { canProcess, reason } = checkWordLimits(words);
      if (!canProcess) {
        setModalType(reason === 'daily_limit' ? 'daily' : 'words');
        setModalOpen(true);
        return;
      }
    }

    setIsProcessing(true);
    try {
      const storedAccess = getStoredAccessCode();
      const accessCode = storedAccess?.code;

      const cleanedText = await cleanDashesWithAI(input, removeEmojisEnabled, isPro, accessCode);
      setOutput(cleanedText);
      addProcessedWords(words);
      if (autoCopyEnabled) {
        await copyToClipboard(cleanedText);
      }
    } catch (error) {
      console.error('Error processing text:', error);
    } finally {
      setIsProcessing(false);
    }
  };

  const copyToClipboard = async (text: string) => {
    try {
      await navigator.clipboard.writeText(text);
      setCopied(true);
      setTimeout(() => setCopied(false), UI.COPY_TIMEOUT);
    } catch (err) {
      console.error('Failed to copy text:', err);
    }
  };

  const handleUnlock = (type: 'pro' | 'credits') => {
    setIsPro(type === 'pro');
    setIsUnlockModalOpen(false);
  };

  const handlePremiumFeature = (feature: string) => {
    if (!isPro) {
      setSelectedFeature(feature);
      setIsPremiumModalOpen(true);
      return;
    }
    
    if (feature === 'Auto-copy') {
      setAutoCopyEnabled(!autoCopyEnabled);
    } else if (feature === 'Remove Emojis') {
      setRemoveEmojisEnabled(!removeEmojisEnabled);
    }
  };

  const handleClipboardPaste = (text: string) => {
    setInput(text);
  };

  const handlePaymentModal = (open: boolean) => {
    startTransition(() => {
      setPaymentModalOpen(open);
    });
  };

  const handleUnlockModal = (open: boolean) => {
    startTransition(() => {
      setIsUnlockModalOpen(open);
    });
  };

  const handlePremiumModal = (open: boolean) => {
    startTransition(() => {
      setIsPremiumModalOpen(open);
    });
  };

  return (
    <div className="min-h-screen flex flex-col">
      <nav className="border-b border-gray-200">
        <div className="nav-container">
          <div className="h-auto sm:h-12 flex flex-col sm:flex-row items-start sm:items-center justify-between py-2 sm:py-0">
            <div className="flex items-center justify-between w-full sm:w-auto">
              <div className="flex items-center">
                <AlignTextLeft className="h-5 w-5 text-indigo-600" style={{ transform: 'scale(-1)' }} />
                <span className="ml-2 text-m font-medium text-gray-900">RemoveDash</span>
              </div>
              <span className="text-sm sm:hidden">
                {wordsUsedToday.toLocaleString()} / {dailyWordLimit.toLocaleString()} words
              </span>
            </div>
            
            <div className="flex items-center justify-between w-full sm:w-auto mt-2 sm:mt-0">
              <span className="hidden sm:inline text-sm mr-4">
                {wordsUsedToday.toLocaleString()} / {dailyWordLimit.toLocaleString()} words today
              </span>
              
              <div className="flex items-center space-x-2">
                {!isPro && (
                  <button
                    onClick={() => handlePaymentModal(true)}
                    className="button-primary text-white bg-gradient-to-r from-purple-500 to-pink-500"
                  >
                    <Sparkles className="h-4 w-4 mr-1.5" />
                    Unlock all features
                  </button>
                )}
                <button
                  onClick={() => handleUnlockModal(true)}
                  className="button-secondary"
                >
                  <Key className="h-3 w-3 mr-2" />
                  {isPro ? 'Manage Access' : 'Enter Code'}
                </button>
              </div>
            </div>
          </div>
        </div>
      </nav>

      <main className="flex-1">
        <div className="container">
          <div className="grid-layout">
            {!isPro && (
              <div className="grid-span-12 text-left">
                <h1 className="text-2xl">
                  Clean Up GPT-Generated Text — Remove Em Dashes, Emojis — & AI Clutter 🤖✨
                </h1>
                <p className="text-xl max-w-2xl">
                  Tired of awkward AI output — whether you wrote it or someone else did? Use RemoveDash to clean up GPT text: fix punctuation ✏️, remove emojis 😂, and ditch those em dashes 😩 — instantly and for free. 🎉
                </p>
              </div>
            )}

            <div className="grid-span-6">
              <div className="mb-2 flex flex-col sm:flex-row items-start sm:items-center justify-between">
                <h2 className="text-lg font-medium text-gray-900 mb-0 sm:mb-0">Input Text</h2>
                <div className="flex items-center space-x-1">
                  <PremiumFeatureButton
                    onClick={() => handlePremiumFeature('Auto-copy')}
                    icon={<Copy className="h-3 w-3" />}
                    className={autoCopyEnabled ? 'button-primary' : 'button-secondary'}
                    isPro={isPro}
                  >
                    Auto-copy
                  </PremiumFeatureButton>
                  <PremiumFeatureButton
                    onClick={() => handlePremiumFeature('Remove Emojis')}
                    icon={<Smile className="h-3 w-3" />}
                    className={removeEmojisEnabled ? 'button-primary' : 'button-secondary'}
                    isPro={isPro}
                  >
                    Remove Emojis
                  </PremiumFeatureButton>
                </div>
              </div>
              <div className="bg-white shadow-lg rounded-lg overflow-hidden border border-gray-200">
                <AutoResizingTextarea
                  value={input}
                  onChange={(e) => {
                    const newText = e.target.value;
                    const newWordCount = newText.trim() ? newText.trim().split(/\s+/).length : 0;
                    
                    if (!isPro && newWordCount > perUseWordLimit) {
                      setModalType('words');
                      setModalOpen(true);
                      return;
                    }
                    
                    setInput(newText);
                    setWordCount(newWordCount);
                  }}
                  className="w-full h-[320px] p-4 text-base border-0 focus:ring-0 resize-none font-sans"
                  placeholder="Paste your text here..."
                  spellCheck="false"
                  autoComplete="off"
                  autoCorrect="off"
                  autoCapitalize="off"
                  maxLength={isPro ? undefined : perUseWordLimit * 7}
                />
                <div className="border-t border-gray-200 bg-gray-50 p-3">
                  <div className="flex items-center justify-between">
                    <div className="text-sm flex items-center space-x-4">
                      {!isPro && (
                        <div className="flex items-center text-gray-500">
                          <Lock className="h-4 w-4 mr-1.5" />
                          <span>{wordCount} / {perUseWordLimit} words</span>
                        </div>
                      )}
                    </div>
                    <button
                      onClick={handleProcess}
                      disabled={isProcessing || !input.trim()}
                      className="button-primary disabled:opacity-50 disabled:cursor-not-allowed"
                    >
                      {isProcessing ? (
                        <>
                          <Clock className="h-4 w-4 mr-2 animate-spin" />
                          Processing...
                        </>
                      ) : (
                        <>
                          <Sparkles className="h-4 w-4 mr-2" />
                          Clean Text
                        </>
                      )}
                    </button>
                  </div>
                </div>
              </div>
            </div>

            <div className="grid-span-6">
              <div className="mb-2 flex items-center justify-between">
                <h2 className="text-lg font-medium text-gray-900 mb-0">Result</h2>
                <button
                  onClick={() => copyToClipboard(output)}
                  disabled={!output || copied}
                  className={copied ? 'button-secondary text-green-600' : 'button-secondary'}
                >
                  {copied ? (
                    <>
                      <Check className="h-3 w-3 mr-1.5" />
                      Copied!
                    </>
                  ) : (
                    <>
                      <Copy className="h-3 w-3 mr-1.5" />
                      Copy
                    </>
                  )}
                </button>
              </div>
              <div className="bg-white rounded-lg overflow-hidden border border-gray-200">
                <AutoResizingTextarea
                  value={output}
                  className="w-full h-[320px] p-4 text-base bg-gray-50 border-0 focus:ring-0 resize-none"
                  placeholder="Cleaned text will appear here..."
                  readOnly
                />
              </div>
            </div>

            {!isPro && (
              <div className="grid-span-12 text-left">
                <div className="">
                  <ProductHuntBadge />
                  
                  <h2 className="text-2xl">Where Is the Em Dash From, Anyway?</h2>
                  <p>You've seen it. You've probably used it—maybe even more than you should have. That long, dramatic pause disguised as punctuation: the em dash.</p>
                  <p>It's not quite a comma. Not quite a colon. Definitely not a semicolon. But somehow, the em dash has become the darling of internet writers, newsletter editors, Medium bloggers, and, let's be honest, AI models like GPT.</p>
                  <p>But where did it come from? And why does it keep showing up in text that feels just a little… off?</p>
                  <h2>A Brief History of the Em Dash</h2>
                  <p>The em dash gets its name from typography: it's the width of a capital "M" in whatever typeface you're using. That's it. That's the origin. Glamorous, right?</p>
                  <p>Its function, though, is a little more slippery. In classical typesetting, the em dash was used to signal a break in thought, a pause longer than a comma but less final than a period. It replaced parentheses, colons, even dialogue tags in literature. Emily Dickinson loved it. So did James Joyce. So do most AI models trained on their work.</p>
                  <h2>Why the Em Dash Is Everywhere Now</h2>
                  <p>We live in the era of conversational tone. You don't write at people anymore, you write with them. Or at least you try to. The em dash fits right into that vibe. It feels casual, flexible, modern.</p>
                  <p>Which is exactly why AI loves it.</p>
                  <p>GPT, Claude, Gemini, and their cohort of text-spitting bots are trained on human writing. And modern human writing is saturated with em dashes. These models, in an effort to sound natural, over-learn the em dash. So when you ask for a product description, email draft, or blog outline, the result often comes loaded with them.</p>
                  <h2>The Danger of Copy → Paste → Publish</h2>
                  <p>AI text isn't bad. In fact, it can be really good. The danger is assuming it's ready.</p>
                  <p>We've all seen it. A colleague pastes a ChatGPT block straight into an email or a blog post and hits send. No edits. No formatting. No thought. Just long sentences, stacked punctuation, and emojis like confetti. It's obvious. It reads like a robot trying to be your quirky assistant and missing the mark.</p>
                  <p>The problem isn't the em dash. It's not the emoji. It's what those things represent: <strong>you didn't even check your work.</strong></p>
                  <p>AI output has tells. Dash overload. Emojis for emotion. Overly polished sentences that never quite land. And once you've seen the pattern, you can't unsee it. It doesn't matter how solid the content is — if it looks like AI and smells like AI and reads like AI, it screams: "I phoned it in."</p>
                  <h2>One Dash Away From a Facepalm</h2>
                  <p>This isn't about sounding more human. It's about not looking like someone who outsourced their brain and forgot to proofread the results.</p>
                  <p>The AI paragraph that tries too hard? You know the one. Dashes in every line. Emojis as punctuation. Sentences that feel like they're one Canva quote template away from a hustle post. It's not a crime. But it is a red flag.</p>
                  <p>Readers can feel the "meh" from two scrolls away. And if your goal was to be taken seriously, or even just be read, you're now one dash away from a facepalm.</p>
                  <h2>Meet RemoveDash: A Tiny Tool That Saves You From Looking Like a Bot</h2>
                  <p>
                    <a
                      href="https://removedash.com"
                      className="underline font-medium text-blue-600 hover:text-blue-800"
                      target="_blank"
                      rel="noopener noreferrer"
                    >
                      RemoveDash.com
                    </a>{" "}
                    is a lightweight, one-click cleanup tool that scrubs the obvious tells: em dashes, random emojis, AI filler formatting — without changing your tone or overwriting your voice.
                  </p>
                  <p>No fluff. No login. No judgment. Just a smarter final pass before you hit send, publish, or paste.</p>
                  <p>Because if you're going to use AI, at least make it look like you gave a damn.</p>
                </div>
              </div>
            )}

            <div className="grid-span-12 mt-16">
              <FAQ isPro={isPro} />
            </div>
          </div>
        </div>

        {!isPro && isProcessing && (
          <ProcessingBanner onUpgrade={() => handlePaymentModal(true)} />
        )}
      </main>

      <Suspense fallback={<LoadingSpinner />}>
        <ComingSoon />
        <Footer />
      </Suspense>

      <Suspense fallback={<LoadingSpinner />}>
        {isPaymentModalOpen && (
          <PaymentModal
            isOpen={isPaymentModalOpen}
            onClose={() => handlePaymentModal(false)}
          />
        )}
        {isUnlockModalOpen && (
          <UnlockModal
            isOpen={isUnlockModalOpen}
            onClose={() => handleUnlockModal(false)}
            onUnlock={handleUnlock}
          />
        )}
        {isPremiumModalOpen && (
          <PremiumFeatureModal
            isOpen={isPremiumModalOpen}
            onClose={() => handlePremiumModal(false)}
            onUpgrade={() => {
              handlePremiumModal(false);
              handlePaymentModal(true);
            }}
            featureName={selectedFeature}
          />
        )}
      </Suspense>

      {showClipboardOverlay && (
        <ClipboardOverlay
          onPaste={handleClipboardPaste}
          onClose={() => setShowClipboardOverlay(false)}
        />
      )}

      <Transition appear show={modalOpen} as={Fragment}>
        <Dialog
          as="div"
          className="relative z-10"
          onClose={() => setModalOpen(false)}
        >
          <Transition.Child
            as={Fragment}
            enter="ease-out duration-300"
            enterFrom="opacity-0"
            enterTo="opacity-100"
            leave="ease-in duration-200"
            leaveFrom="opacity-100"
            leaveTo="opacity-0"
          >
            <div className="fixed inset-0 bg-black/25 backdrop-blur-sm" />
          </Transition.Child>

          <div className="fixed inset-0 overflow-y-auto">
            <div className="flex min-h-full items-center justify-center p-4 text-center">
              <Transition.Child
                as={Fragment}
                enter="ease-out duration-300"
                enterFrom="opacity-0 scale-95"
                enterTo="opacity-100 scale-100"
                leave="ease-in duration-200"
                leaveFrom="opacity-100 scale-100"
                leaveTo="opacity-0 scale-95"
              >
                <Dialog.Panel className="w-full max-w-md transform overflow-hidden rounded-2xl bg-white p-6 text-left align-middle shadow-xl transition-all">
                  <Dialog.Title
                    as="h3"
                    className="text-lg font-medium leading-6 text-gray-900 flex items-center"
                  >
                    <AlertCircle className="h-5 w-5 mr-2 text-red-500" />
                    Word Limit Reached
                  </Dialog.Title>
                  <div className="mt-2">
                    <p className="text-gray-600">
                      {modalType === 'daily'
                        ? `You've reached your daily limit of ${dailyWordLimit} words.`
                        : `Free accounts can only process ${perUseWordLimit} words at a time.`}
                    </p>
                  </div>

                  <div className="mt-4">
                    <button
                      type="button"
                      className="button-primary"
                      onClick={() => {
                        setModalOpen(false);
                        handlePaymentModal(true);
                      }}
                    >
                      <Sparkles className="h-4 w-4 mr-2" />
                      Upgrade to Pro
                    </button>
                  </div>
                </Dialog.Panel>
              </Transition.Child>
            </div>
          </div>
        </Dialog>
      </Transition>
    </div>
  );
}

function App() {
  const location = useLocation();
  
  return (
    <Routes>
      <Route path="/success" element={<SuccessPage />} />
      <Route path="/terms" element={<TermsPage />} />
      <Route path="/privacy" element={<PrivacyPage />} />
      <Route path="/login" element={<LoginPage />} />
      <Route 
        path="/admin" 
        element={
          <ProtectedRoute>
            <AdminPage />
          </ProtectedRoute>
        } 
      />
      <Route path="/" element={<MainApp />} />
    </Routes>
  );
}

export default App;